/*
		Room.cpp : implementation file

        Copyright (C) 2000 Patrick Dughi

        This software may be used and distributed according to the terms
        of the GNU Public License, incorporated herein by reference, located
		as the file gpl.txt in this same directory.

        The author may be reached at dughi@imaxx.net.
.		
		Room class structure and related functions (load/write/etc)
*/
#pragma warning(disable:4786)

#include <vector>
#include <string>
#include <map>
#include <stdlib.h>
#include <ctype.h>
#include "stdafx.h"
#include "Cbreak.h"
#include "Room.h"
#include "utils.h"

#pragma warning(disable:4786)
using namespace std;

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Room::Room()
{
  int found;

  for(found=0;found < NUM_OF_DIRS;found++) {
	  direction_data[found].target_room = -1;
	  direction_data[found].key_vnum = 0;
  }
  sector_type=room_flags=0; 
}

Room::~Room()
{

}


Ex_Desc_Data * Room::get_exdata(int x)
{
	if(exdesc.size() > x ) { 
		return &(exdesc[x]); 
	} 
	return NULL;
}

int Room::get_num_exdata()
{
  return (exdesc.size());
}

Ex_Desc_Data *Room::get_exdata(string x)
{
  int i;
  for(i=0;i<exdesc.size();i++) {
	  if(exdesc[i].keyword == x) {
	    return &(exdesc[i]);
	  }
  }
  return NULL;
}

bool Room::delete_exdata(int x)
{
	if(exdesc.size() <= x) {
		return FALSE;
	}
	// our iterator is implied. churrah.
	exdesc.erase(exdesc.begin() + x );
	return TRUE;
}

bool Room::delete_exdata(string x)
{
	vector<Ex_Desc_Data>::iterator exd;
  for(exd=exdesc.begin();exd != exdesc.end();exd++) {
	  if(exd->keyword == x) {
		  exdesc.erase(exd);
		  return TRUE;
	  }
  }
  return FALSE;

}




// we assume cfile is open, and correctly placed.
void frigfrag(string *sex);
bool Room::write_room(CircleFile *cfile)
{
  string obuf;
  string temp;
  char buf[33];
  int i;
  Ex_Desc_Data *tex;

  obuf = iprintf("#%d\n",number);

  if(name.size()) {
	  obuf += name;
  } else {
	  obuf += "Untitled";
  }
  obuf += "~\n";

  if(description.size()) {
	  temp=description;
	  frigfrag(&temp);
    obuf += iprintf("%s",temp.c_str());
  }else{
    obuf+= "Empty Room";
  }
  obuf += iprintf("\n~\n%d ",zone);
  bitvector2ascii(buf);
  obuf += iprintf("%s %d\n",buf,sector_type);
   
  // format is d{i}\n
  for(i=0; i < NUM_OF_DIRS;i++) {
	  if(direction_data[i].target_room >= 0) {
		  obuf += iprintf("D%d\n",i);
		  obuf += iprintf("%s\n~\n",direction_data[i].look.c_str());
		  obuf += iprintf("%s~\n",direction_data[i].keyword.c_str());
		  obuf += iprintf("%d %d %d\n",direction_data[i].exit_bits,
			direction_data[i].key_vnum,direction_data[i].target_room);	
	  }
  }
  for(i=0;i<get_num_exdata();i++) {
	  tex = get_exdata(i);
	  obuf += "E\n";
	  obuf += iprintf("%s~\n",tex->keyword.c_str());
	  temp=tex->desc.c_str();
	  frigfrag(&temp);
	  obuf += iprintf("%s\n~\n",temp.c_str());
  }

  obuf+= "S\n";

  return(cfile->write_line(&obuf)); 
  
}

void Room::bitvector2ascii(char *output)
{
  int i, j = 0;
  char *flags="abcdefghijklmnopqrstuvwxyzABCDEF";

  for (i = 0; flags[i] != '\0'; i++) {
	  if (room_flags & (1 << i)) {
        output[j++] = flags[i];
    }
	}

  if (j == 0) {
    output[j++] = '0';
  }

  output[j]='\0';
  return;
}

/* number should be supplied, zone too, from the filename...sorta */

int Room::load_room(CircleFile *cf)
{
  map<int,Zone_Data>::const_iterator zi;
	Ex_Desc_Data *new_desc;
	Room_Dir_Data *new_dir;
  int found=0;
  extern World the_world;
  string temp;
  int bad=0;

  /* have to verify that each room is _in_ a zone */
  for(zi=the_world.zone_list.begin();zi!=the_world.zone_list.end() && !found;zi++) {

	  if(zi->first * 100 <= number && zi->second.top >= number) {
		  found=1;
		}

  }

	if(!found) {
		TRACE0(iprintf("Room %d not found in a zone.\n",number));
		bad=1;
	}
    zone = cf->the_zone;

	cf->tilde_read(&temp);
	name = temp;  
	cf->tilde_read(&temp);
	description = temp;
	cf->get_line(&temp);


  room_flags = ascii2bitvector((temp.substr(temp.find(" ",1)+1,
						temp.substr(temp.find(" ",1)+1).find(" ",1)).c_str()));
  sector_type = atoi(temp.substr(temp.rfind(" ")+1).c_str());

	found = 0;
	cf->get_line(&temp);
	while(!temp.empty() && temp.at(0) != 'S') {
		switch(temp.at(0)) {
			case 'E':
				new_desc = new Ex_Desc_Data;
				cf->tilde_read(&temp);
				new_desc->keyword = temp;
				cf->tilde_read(&temp);
				new_desc->desc = temp;
				exdesc.push_back(*new_desc);
				break;
			case 'D':
	
				new_dir = new Room_Dir_Data;
        found = temp.at(1) - '0';

				cf->tilde_read(&temp);
				new_dir->look = temp;
				cf->tilde_read(&temp);
				new_dir->keyword = temp;
				cf->get_line(&temp);
				new_dir->exit_bits = atoi(temp.substr(0,temp.find(" ") - 1).c_str());
				new_dir->key_vnum = atoi(temp.substr(temp.find(" ")+1, temp.rfind(" ")-1).c_str());
				new_dir->target_room = atoi(temp.substr(temp.rfind(" ")+1).c_str());
				direction_data[found] = *new_dir;
				if(found >= 7) {
					the_world.more_doors = 1;
				}

				break;
			default:
				TRACE0(iprintf("A room default case?..got %s\n",temp.c_str()));
				break;
		}
		cf->get_line(&temp);
	}
   
	if(bad) {
		return false;
	} else {
      return true;
	}
}

int Room::ascii2bitvector(const char *input)
{
  int i;
	int bitvector=0;

	if(isdigit(input[0])) {
		return (atoi(input));
	}

	for(i=0;i<strlen(input);i++) {
		if(input[i] >= 97 && input[i] <= 122) {
			bitvector |= 1 << (input[i] - 'a');
		} else {
			bitvector |= 1 << ((input[i] - 'A')+26);
		}
	}
	return bitvector;
}

void Room::add_exdata(string keywords, string desc)
{
    Ex_Desc_Data *exd;
	delete_exdata(keywords);
	exd = new Ex_Desc_Data;
	exd->desc = desc;
	exd->keyword = keywords;
    exdesc.push_back(*exd); 
}

int Room::add_zcmd(Zone_Cmd_Data *new_zcmd)
{
  zonecmd.push_back(*new_zcmd);
  return TRUE;
}

void Room::SwitchZcmds(int from, int to)
{
  Zone_Cmd_Data *zc;

  zc = new Zone_Cmd_Data;

  zc->cmd = zonecmd[from].cmd;
  zc->depend = zonecmd[from].depend;
  zc->arg[0] = zonecmd[from].arg[0];
  zc->arg[1] = zonecmd[from].arg[1];
  zc->arg[2] = zonecmd[from].arg[2];

  zonecmd.erase(zonecmd.begin()+from);
  if(to > from) {
    zonecmd.insert(zonecmd.begin()+to-1,*zc);
  } else {
	zonecmd.insert(zonecmd.begin()+to,*zc);
  }
}
